#include "declarations.h"

#ifdef USE_PYTHON
#include "bindings.h"
#include "macro_language.h"

int get_file_number()
{
   int rval;
   rval = g_list_index(files_list, (gpointer)cur_file);
   return(rval);
}

int change(int file_no)
{
   GdsFile *file;
   int success = FALSE;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   if(file)
   {
      file_change(NULL, file);
      success = TRUE;
   }
   return(success);   
}

int get_files()
{
   return(total_files);
}

int get_file_length(int file_no)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   g_return_val_if_fail(file != NULL, -1);
   g_return_val_if_fail(GDS_IS_FILE(file), -1);
   return(GTK_EXTEXT(file->text)->length);
}

int get_file_changed(int file_no)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   g_return_val_if_fail(file != NULL, -1);
   g_return_val_if_fail(GDS_IS_FILE(file), -1);
   if(GDS_EDITOR(file->text)->changed) return(TRUE);
   return(FALSE);
}

int get_file_tab_width(int file_no)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   g_return_val_if_fail(file != NULL, -1);
   g_return_val_if_fail(GDS_IS_FILE(file), -1);
   return(GTK_EXTEXT(file->text)->default_tab_width);
}

/* These will probably leak a great deal */

char *get_full_filename(int file_no)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   g_return_val_if_fail(file != NULL, NULL);
   g_return_val_if_fail(GDS_IS_FILE(file), NULL);
   return(g_strdup(file->filename));
}

char *get_filename(int file_no)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   g_return_val_if_fail(file != NULL, NULL);
   g_return_val_if_fail(GDS_IS_FILE(file), NULL);
   return(get_file_from_filename(file->filename));
}

char *get_path(int file_no)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   g_return_val_if_fail(file != NULL, NULL);
   g_return_val_if_fail(GDS_IS_FILE(file), NULL);
   return(get_path_from_filename(file->filename));
}

char *get_file(int file_no)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   g_return_val_if_fail(file != NULL, NULL);
   g_return_val_if_fail(GDS_IS_FILE(file), NULL);
   return(get_file_wo_extension(file->filename));
}

int has_selection()
{
   gint value;
   GtkEditable *editable;
   g_return_val_if_fail(cur_file, 0);
   editable = GTK_EDITABLE(cur_file->text);
   value = (int)editable->has_selection;
   return(value);   
}

int selection_start()
{
   int value;
   GtkEditable *editable;
   g_return_val_if_fail(cur_file, -1);
   editable = GTK_EDITABLE(cur_file->text);
   value = (int)editable->selection_start_pos;
   return(value);
}

int selection_end()
{
   int value;
   GtkEditable *editable;
   g_return_val_if_fail(cur_file, -1);
   editable = GTK_EDITABLE(cur_file->text);
   value = (int)editable->selection_end_pos;
   return(value);
}

void move(int relative_movement)
{
   int index;
   g_return_if_fail(cur_file);
   index = gtk_extext_get_position(GTK_EDITABLE(cur_file->text));
   index += relative_movement;
   g_return_if_fail(index >= 0);
   gtk_extext_set_position(GTK_EDITABLE(cur_file->text), index);
}

void move_to(int index)
{
   g_return_if_fail(cur_file);
   g_return_if_fail(index >= 0);
   gtk_extext_set_position(GTK_EDITABLE(cur_file->text), index);
}

int current_position(void)
{
   g_return_val_if_fail(cur_file, -1);
   return gtk_extext_get_position(GTK_EDITABLE(cur_file->text));
}

void move_to_line (int index)
{
   g_return_if_fail(cur_file);
   g_return_if_fail(index >= 1);
   gtk_extext_set_line(GTK_EXTEXT(cur_file->text), index-1);
}

void move_lines(int relative_movement)
{
   g_return_if_fail(cur_file);
   int index = gtk_extext_get_line(GTK_EXTEXT(cur_file->text));
   gtk_extext_set_line(GTK_EXTEXT(cur_file->text),index+relative_movement);
}

int current_line(void)
{
   g_return_val_if_fail(cur_file, -1);
   return gtk_extext_get_line(GTK_EXTEXT(cur_file->text)+1);
}

char* get_text(int begin,int end)
{
   g_return_val_if_fail(cur_file, NULL);
   return gtk_extext_get_text(GTK_EXTEXT(cur_file->text),begin,end);
}

char get_current_char(void)
{
   int index;
   g_return_val_if_fail(cur_file, -1);
   index = gtk_extext_get_position(GTK_EDITABLE(cur_file->text));
   return GTK_EXTEXT_INDEX(GTK_EXTEXT(cur_file->text), index);
}

char get_next_char()
{
   char c;
   int index;
   g_return_val_if_fail(cur_file, -1);
   index = gtk_extext_get_position(GTK_EDITABLE(cur_file->text));
   c = GTK_EXTEXT_INDEX(GTK_EXTEXT(cur_file->text),index);
   index++;
   gtk_extext_set_position(GTK_EDITABLE(cur_file->text), index);
   return(c);
}

int buffer_size()
{
   g_return_val_if_fail(cur_file, -1);
   return gtk_extext_get_length(GTK_EXTEXT(cur_file->text));
}

void insert_in(int file_no, int pos, char* text)
{
   GtkEditable *editable;
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   g_return_if_fail(file != NULL);
   g_return_if_fail(text != NULL);
   editable = GTK_EDITABLE(file->text);
   gtk_editable_insert_text(editable, text, strlen(text), (gint *)&pos);
}

void insert(char* text)
{
   GtkEditable *editable;
   g_return_if_fail(text != NULL);
   editable = GTK_EDITABLE(cur_file->text);
   gtk_editable_insert_text(editable, text, strlen(text), (gint *)&editable->current_pos);
}

void delete_text(int start, int end)
{
   g_return_if_fail(cur_file);
   delete_text_in(get_file_number(), start, end);
}

void delete_text_in(int file_no, int start, int end)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   g_return_if_fail(file != NULL);
   gtk_editable_delete_text(GTK_EDITABLE(file->text), start, end);
}

void forward_delete(int n)
{
   int cur_pos;
   g_return_if_fail(cur_file);
   cur_pos = current_position();
   gtk_editable_delete_text(GTK_EDITABLE(cur_file->text), cur_pos, cur_pos + n);
}

void backward_delete(int n)
{
   int cur_pos;
   g_return_if_fail(cur_file);
   cur_pos = current_position();
   gtk_editable_delete_text(GTK_EDITABLE(cur_file->text),cur_pos-n,cur_pos);
}

void undo()
{
   g_return_if_fail(cur_file);
   undo_last_real(cur_file);
}

void undo_last(int file_no)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   undo_last_real(file);
}

void redo()
{
   g_return_if_fail(cur_file);
   redo_last_real(cur_file);
}

void redo_last(int file_no)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   redo_last_real(file);
}

void cut()
{
   g_return_if_fail(cur_file);
   cut_text_real(cur_file);
}

void cut_text(int file_no)
{
   GtkEditable *editable;
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   cut_text_real(file);
}

void copy()
{
   g_return_if_fail(cur_file);
   copy_text_real(cur_file);
}

void copy_text(int file_no)
{
   GtkEditable *editable;
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   copy_text_real(file);
}

void paste()
{
   g_return_if_fail(cur_file);
   paste_text_real(cur_file);
}

void paste_text(int file_no)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   paste_text_real(cur_file);
}

void clear()
{
   g_return_if_fail(cur_file);
   clear_text_real(cur_file);
}

void clear_text(int file_no)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   clear_text_real(file);
}

void clear_entry()
{
   clear_entry_real();
}

void comment()
{
   g_return_if_fail(cur_file);
   comment_selection_real(cur_file);
}

void comment_selection(int file_no)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   comment_selection_real(file);
}

void select_area(int from, int to)
{
   g_return_if_fail(cur_file);
   gtk_extext_set_position(GTK_EDITABLE(cur_file->text), to);
   gtk_editable_select_region(GTK_EDITABLE(cur_file->text), from, to);
   gtk_widget_grab_focus(cur_file->text);
}

void select_text(int file_no, int from, int to)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   g_return_if_fail(file);
   gtk_extext_set_position(GTK_EDITABLE(cur_file->text), to);
   gtk_editable_select_region(GTK_EDITABLE(file->text), from, to);
   gtk_widget_grab_focus(cur_file->text);
}

void select_current()
{
   g_return_if_fail(cur_file);
   select_current_real(cur_file);
}

void select_all()
{
   g_return_if_fail(cur_file);
   select_all_text_real(cur_file);
}

void select_all_text(int file_no)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   select_all_text_real(file);
}

void highlight_syntax(int file_no)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   highlight_syntax_real(file);
}

void new_window()
{
   new_window_cb(NULL, NULL);
}

void new_file(char *filename)
{
   new_file_real(filename);
}

void open_file(char *filename)
{
   open_file_real(filename);
}

void close_file(int file_no)
{
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
   close_file_real(file);
}

void save_all_files()
{
   save_all_files_cb(NULL, NULL);
}

void close_all_files()
{
   close_all_files_cb(NULL, NULL);
}

int find(char *text)
{
	g_return_val_if_fail(cur_file, FALSE);
   return(find_in(get_file_number(), text));
}

int find_in(int file_no, char *text)
{
   int success = FALSE;
   GtkExTextMatch m;
   GtkEditable *editable;
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
	g_return_val_if_fail(cur_file, FALSE);
   editable = GTK_EDITABLE(file->text);
   success = (int)find_text(GTK_EXTEXT(cur_file->text), editable->current_pos, GTK_EXTEXT(cur_file->text)->length, text, FALSE, FALSE, TRUE, &m);
   gtk_widget_grab_focus(python_entry);
   return(success);
}

int find_regex(char *text)
{
	g_return_val_if_fail(cur_file, FALSE);
   return(find_regex_in(get_file_number(), text));
}

int find_regex_in(int file_no, char *text)
{
   int success = FALSE;
   GtkExTextMatch m;
   GtkEditable *editable;
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
	g_return_val_if_fail(cur_file, FALSE);
   editable = GTK_EDITABLE(file->text);
   success = (int)find_text(GTK_EXTEXT(cur_file->text), editable->current_pos, GTK_EXTEXT(cur_file->text)->length, text, FALSE, TRUE, FALSE, &m);
   gtk_widget_grab_focus(python_entry);
   return(success);
}

int replace(char *search_text, char *replacer)
{
	g_return_val_if_fail(cur_file, FALSE);
   return(replace_in(get_file_number(), search_text, replacer));
}

int replace_in(int file_no, char *search_text, char *replacer)
{
   int success = FALSE;
   GtkExTextMatch m;
   GtkEditable *editable;
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
	g_return_val_if_fail(cur_file, FALSE);
   editable = GTK_EDITABLE(file->text);
   if(find_text(GTK_EXTEXT(file->text), editable->current_pos, GTK_EXTEXT(file->text)->length, search_text, TRUE, FALSE, TRUE, &m))
   {
      replace_text(GTK_EXTEXT(file->text), replacer);
      success = TRUE;
   }
   gtk_widget_grab_focus(python_entry);
   return(success);
}

int replace_all(char *search_text, char *replacer)
{
	g_return_val_if_fail(cur_file, FALSE);
   return(replace_all_in(get_file_number(), search_text, replacer));
}

int replace_all_in(int file_no, char *search_text, char *replacer)
{
   int success = FALSE;
   GtkExTextMatch m;
   GtkEditable *editable;
   GdsFile *file;
   file = (GdsFile *)g_list_nth_data(files_list, file_no);
	g_return_val_if_fail(cur_file, FALSE);
	success = (int)replace_all_text(GTK_EXTEXT(file->text), search_text, replacer, TRUE, GTK_EXTEXT(file->text)->length);
   return(success);
}

#endif