#include "declarations.h"

#ifdef USE_PYTHON
#include "Python.h"
#include "macro_language.h"
#include "bindings.h"

GList *scriptsmenu_items = NULL;

extern "C" { void initcc(); }

void initialize_macro_language(void)
{
   Py_Initialize();
   PyImport_AddModule("cc");
   initcc();
   PyRun_SimpleString("import cc");
}

void execute_macro_string(char* string)
{
  PyRun_SimpleString(string);
}

void autoexec_script(void)
{
   gchar* filename;
   FILE* autoexec_file;
   filename= g_strconcat(getenv("HOME"),"/.",PACKAGE,"/autoexec.py",NULL);
   if(autoexec_file = fopen(filename,"r"))
     PyRun_SimpleFile(autoexec_file,"autoexec.py");
   g_free(filename);
   if(autoexec_file) fclose(autoexec_file);
}

gint build_menu_from_scripts_dir(gchar *path, GtkWidget *parent)
{
   DIR *dir_list;
   struct dirent *dir;
   struct stat st;
   gchar filename[384];
   gchar *label;
   gchar *text;
   GtkWidget *menu;
   GtkWidget *menu_item;
   gint items = 0;
   
   dir_list = opendir(path);
   if(!dir_list) return(0);
   while((dir = readdir(dir_list)))
   {
      text = g_strconcat(path, "/", dir->d_name, NULL);
	   stat(text, &st);
      g_free(text);
      if(!S_ISDIR(st.st_mode))
      {
         if(strstr(dir->d_name, ".py"))
         {
            label = get_file_wo_extension(dir->d_name);
            menu_item = gtk_menu_item_new_with_label(label);
            gtk_menu_append(GTK_MENU(parent), menu_item);
            text = g_strconcat(path, "/", dir->d_name, NULL);
            gtk_object_set_data(GTK_OBJECT(menu_item), "path", text);
            gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(script_run_cb), (gpointer)text);
            gtk_widget_show(menu_item);
            scriptsmenu_items = g_list_append(scriptsmenu_items, (gpointer)menu_item);
            g_free(label);
            items++;
         }
      }
      else if(strcmp(dir->d_name, ".") && strcmp(dir->d_name, ".."))
      {
         menu_item = gtk_menu_item_new_with_label(dir->d_name);
         gtk_menu_append(GTK_MENU(parent), menu_item);
         scriptsmenu_items = g_list_append(scriptsmenu_items, (gpointer)menu_item);
         gtk_widget_show(menu_item);
         menu = gtk_menu_new();
         gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), menu);
         scriptsmenu_items = g_list_append(scriptsmenu_items, (gpointer)menu);
         gtk_widget_show(menu);
         text = g_strconcat(path, "/", dir->d_name, NULL);
         items += build_menu_from_scripts_dir(text, menu);
         g_free(text);
      }
   }
   closedir(dir_list);
   return(items);
}

void build_scripts_menu()
{
   gchar path[384];
   g_snprintf(path, sizeof(path), PREFIX "/share/" PACKAGE "/scripts", getenv("HOME"));
   build_menu_from_scripts_dir(path, scriptsmenu);
   g_snprintf(path, sizeof(path), "%s/." PACKAGE "/scripts", getenv("HOME"));
   build_menu_from_scripts_dir(path, scriptsmenu);
   return;
}

void script_run_cb(GtkWidget *widget, gchar *filename)
{
   FILE *file;
   //g_return_if_fail(cur_file);
   g_print("Executing Python script: %s\n", filename);
   file = fopen(filename, "r");
   if(!file) return;
   PyRun_SimpleFile(file, filename);
   fclose(file);
   return;
}

void regenerate_scripts_cb(GtkWidget *widget, gpointer data)
{
   GList *current;
   gpointer text;
   current = scriptsmenu_items = g_list_reverse(scriptsmenu_items);
   while(current)
   {
      text = gtk_object_get_data(GTK_OBJECT(current->data), "path");
      if(text) g_free(text);
      if(GTK_IS_WIDGET(current->data))
         gtk_widget_destroy(GTK_WIDGET(current->data));
      current = current->next;
      g_print("killing item\n");
   }
   g_list_free(scriptsmenu_items);
   scriptsmenu_items = NULL;
   build_scripts_menu();
}

void call_python_function_cb(GtkWidget *widget, gpointer *function)
{
   PyObject* result;
   g_assert(PyCallable_Check((PyObject*)function));
   result = PyObject_CallObject((PyObject*)function,NULL);
   Py_XDECREF(result);
}

PyObject* add_to_scripts_menu(PyObject* self,PyObject* args)
{
  PyObject *callback;
  GtkWidget* menu_item;
  char* label;
  if (!PyArg_ParseTuple(args, "sO", &label,&callback))
    return NULL;
  if (!PyCallable_Check(callback)) {
    PyErr_SetString(PyExc_TypeError, "add_to_menu argument must be callable");
    return NULL;
  }
  Py_INCREF(callback); /* possible referennce leak */
  menu_item = gtk_menu_item_new_with_label(label);
  gtk_menu_append(GTK_MENU(scriptsmenu), menu_item);
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(call_python_function_cb), (gpointer)callback);
  scriptsmenu_items = g_list_append(scriptsmenu_items, (gpointer)menu_item);
  gtk_widget_show(menu_item);
  Py_INCREF(Py_None);
  /* fixme: shouln't we free label ? */
  return Py_None;
}

#include "bindings_wrap.C"

#endif
