#include "declarations.h"

void gds_file_ask_focus(GtkWidget *widget, GdsFile *gds_file);
void undo_changed(GtkWidget *widget, GdsFile *gds_file);
void gds_file_focus_with_raise(GtkWidget *widget, GdsFile *gds_file);

GdsFile *create_file(gchar *name)
{
   GdsFile *new_file;
   GtkWidget *label;
   gchar cwd[256];
   gchar *path;
   gchar *file;
   gchar *full;

   if(!general_preferences.use_gtk_theme)
   {
      extext_style->text[GTK_STATE_NORMAL] = general_preferences.fg;
      extext_style->bg[GTK_STATE_NORMAL] = general_preferences.bg;
      extext_style->text[GTK_STATE_SELECTED] = general_preferences.s_fg;
      extext_style->bg[GTK_STATE_SELECTED] = general_preferences.s_bg;
      extext_style->bg_pixmap[GTK_STATE_NORMAL] = NULL;
   }
   getcwd(cwd, 256);
   path = get_path_from_filename(name);
   file = get_file_from_filename(name);
   if(!path) full = g_strconcat(cwd, "/", file, NULL);
   else full = g_strconcat(path, "/", file, NULL);
   new_file = (GdsFile *)gds_file_new(full);
   g_free(path);
   g_free(full);
   gtk_signal_connect(GTK_OBJECT(new_file->tear_away), "window_focused", GTK_SIGNAL_FUNC(gds_file_ask_focus), new_file);
   gtk_signal_connect(GTK_OBJECT(new_file->tear_away), "attached", GTK_SIGNAL_FUNC(file_change), new_file);
   gtk_signal_connect(GTK_OBJECT(new_file->text), "undo_changed", GTK_SIGNAL_FUNC(undo_changed), new_file);
   gtk_signal_connect(GTK_OBJECT(new_file->text), "move_to_row", GTK_SIGNAL_FUNC(file_pos_changed), 0);
   gtk_signal_connect(GTK_OBJECT(new_file->text), "move_to_column", GTK_SIGNAL_FUNC(file_pos_changed), 0);
   gtk_widget_set_style(new_file->text, extext_style);
   gtk_widget_show(GTK_WIDGET(new_file));
   files_list = g_list_append(files_list, (gpointer)new_file);
   label = gtk_label_new(file);
   gtk_widget_show(label);
   gtk_notebook_append_page(GTK_NOTEBOOK(files_book), new_file->tear_away, label);
   gtk_signal_connect(GTK_OBJECT(new_file->menu_item), "activate", GTK_SIGNAL_FUNC(gds_file_focus_with_raise), new_file);
   gtk_menu_append(GTK_MENU(windowmenu), new_file->menu_item);
   cur_page = g_list_length(GTK_NOTEBOOK(files_book)->children)-1;
   total_files++;
   gtk_notebook_set_page(GTK_NOTEBOOK(files_book), cur_page);
   set_notebook_page(GTK_NOTEBOOK(files_book), NULL, cur_page, (GdsFile *)g_list_nth_data(files_list, cur_page));
   new_file->props.auto_indent = general_preferences.auto_indent;
   new_file->props.use_spaces = general_preferences.use_spaces;
   new_file->props.spaces = general_preferences.spaces;
   GTK_EXTEXT(new_file->text)->default_tab_width = general_preferences.spaces;
   g_free(file);
   gtk_widget_realize(new_file->text);
   gds_editor_set_font(GDS_EDITOR(new_file->text), extext_style->font);
   gds_editor_set_colors(GDS_EDITOR(new_file->text), &extext_style->text[0], &extext_style->bg[0]);
   return(new_file);
}

void gds_file_ask_focus(GtkWidget *widget, GdsFile *gds_file)
{
   file_change(widget, gds_file);
   gds_editor_set_colors(GDS_EDITOR(gds_file->text), &extext_style->text[GTK_STATE_NORMAL], &extext_style->bg[GTK_STATE_NORMAL]);
}

void gds_file_focus_with_raise(GtkWidget *widget, GdsFile *gds_file)
{
   file_change(widget, gds_file);
   if(GDS_TEAR_AWAY(gds_file->tear_away)->torn)
      gdk_window_raise(GDS_TEAR_AWAY(gds_file->tear_away)->new_parent->window);
}

void undo_changed(GtkWidget *widget, GdsFile *gds_file)
{
   if(gtk_extext_undo_is_empty(GTK_EXTEXT(gds_file->text)))
   {
      gtk_widget_set_sensitive(edit_menu_undo, FALSE);
      gtk_widget_set_sensitive(main_toolbar_undo, FALSE);
   }
   else
   {
      gtk_widget_set_sensitive(edit_menu_undo, TRUE);
      gtk_widget_set_sensitive(main_toolbar_undo, TRUE);
   }
   if(gtk_extext_redo_is_empty(GTK_EXTEXT(gds_file->text)))
   {
      gtk_widget_set_sensitive(edit_menu_redo, FALSE);
      gtk_widget_set_sensitive(main_toolbar_redo, FALSE);
   }
   else
   {
      gtk_widget_set_sensitive(edit_menu_redo, TRUE);
      gtk_widget_set_sensitive(main_toolbar_redo, TRUE);
   }
   gtk_signal_emit_by_name(GTK_OBJECT(gds_file->text), "move_to_column", 0);
}