#include "declarations.h"
#include "macro_language.h"

static void dnd_drop(GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *selection_data, guint info, guint time, gpointer data);

enum { TARGET_URI_LIST };
static GtkTargetEntry drop_types[] =
{
   { "text/uri-list", 0, TARGET_URI_LIST }
};
static gint n_drop_types = sizeof(drop_types) / sizeof(drop_types[0]);

GtkWidget *main_toolbar;
GtkWidget *main_toolbar_new;
GtkWidget *main_toolbar_open;
GtkWidget *main_toolbar_close;
GtkWidget *main_toolbar_save;
GtkWidget *main_toolbar_saveas;
GtkWidget *main_toolbar_undo;
GtkWidget *main_toolbar_redo;
GtkWidget *main_toolbar_cut;
GtkWidget *main_toolbar_copy;
GtkWidget *main_toolbar_paste;
GtkWidget *main_toolbar_properties;

#ifdef USE_PYTHON
GtkWidget *python_combo;
GtkWidget *python_entry;
GList *python_history = NULL;

static void execute_macro_cb(GtkWidget* widget, gpointer data)
{
   gchar *text;
   if(widget == python_entry)
      gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "activate");
   text = gtk_entry_get_text(GTK_ENTRY(python_entry));
   if(strlen(text)>0)
   {
      execute_macro_string(text);
      add_string_to_file("command.hist", text);
      command_combo_add_string(text);
   }
}

static void check_macro_button_cb(GtkWidget* widget, GtkWidget *target)
{
   gchar *text;
   text = gtk_entry_get_text(GTK_ENTRY(python_entry));
   if(strlen(text) > 0)
      gtk_widget_set_sensitive(target, TRUE);
   else
      gtk_widget_set_sensitive(target, FALSE);
}

void command_combo_set_history(GList *history)
{
   if(history == NULL) return;
   python_history = history;
   gtk_combo_set_popdown_strings(GTK_COMBO(python_combo), python_history);
   gtk_entry_set_text(GTK_ENTRY(python_entry), "");
}

gboolean command_combo_add_string(gchar *string)
{
   gint pos;
   gpointer temp;
   GList *ptr;
   gboolean success = FALSE;
   pos = check_list_for_string(python_history, string);
   if(pos)
   {
      temp = g_list_nth_data(python_history, pos-1);
      python_history = g_list_remove(python_history, temp);
   }
   else
   {
      temp = g_new(char, strlen(string)+1);
      strcpy((gchar *)temp, string);
   }

   python_history = g_list_prepend(python_history, temp);

   while(g_list_length(python_history) > general_preferences.history)
   {
      ptr = g_list_nth(python_history, g_list_length(python_history)-1);
      if(ptr)
      {
         temp = ptr->data;
         python_history = g_list_remove(python_history, ptr->data);
         g_free(ptr->data);
      }
   }
   gtk_combo_set_popdown_strings(GTK_COMBO(python_combo), python_history);
   return(success);
}   
#endif

void make_toolbar()
{
   GtkWidget *pixmapwid;
   GtkWidget *label;
   GtkWidget *command_box;
   GtkWidget *macro_button;
   
   main_toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
   gtk_toolbar_set_button_relief(GTK_TOOLBAR(main_toolbar), GTK_RELIEF_NONE);
   gtk_widget_show(main_toolbar);

   pixmapwid = gnome_stock_pixmap_widget_at_size(window, GNOME_STOCK_PIXMAP_NEW, 21, 21);
   main_toolbar_new = gtk_toolbar_append_item(GTK_TOOLBAR(main_toolbar), 0, "New source file", 0, pixmapwid, GTK_SIGNAL_FUNC(new_file_cb), 0); 
   pixmapwid = gnome_stock_pixmap_widget_at_size(window, GNOME_STOCK_PIXMAP_OPEN, 21, 21);
   main_toolbar_open = gtk_toolbar_append_item(GTK_TOOLBAR(main_toolbar), 0, "Open source file", 0, pixmapwid, GTK_SIGNAL_FUNC(open_file_cb), 0); 
   pixmapwid = gnome_stock_pixmap_widget_at_size(window, GNOME_STOCK_PIXMAP_CLOSE, 21, 21);
   main_toolbar_close = gtk_toolbar_append_item(GTK_TOOLBAR(main_toolbar), 0, "Close source file", 0, pixmapwid, GTK_SIGNAL_FUNC(close_file_cb), 0); 
   gtk_toolbar_append_space(GTK_TOOLBAR(main_toolbar));
   pixmapwid = gnome_stock_pixmap_widget_at_size(window, GNOME_STOCK_PIXMAP_SAVE, 21, 21);
   main_toolbar_save = gtk_toolbar_append_item(GTK_TOOLBAR(main_toolbar), 0, "Save source file", 0, pixmapwid, GTK_SIGNAL_FUNC(save_file_cb), 0); 
   pixmapwid = gnome_stock_pixmap_widget_at_size(window, GNOME_STOCK_PIXMAP_SAVE_AS, 21, 21);
   main_toolbar_saveas = gtk_toolbar_append_item(GTK_TOOLBAR(main_toolbar), 0, "Save source file with new name", 0, pixmapwid, GTK_SIGNAL_FUNC(save_file_as_cb), 0); 
   gtk_toolbar_append_space(GTK_TOOLBAR(main_toolbar));
   pixmapwid = gnome_stock_pixmap_widget_at_size(window, GNOME_STOCK_PIXMAP_UNDO, 21, 21);
   main_toolbar_undo = gtk_toolbar_append_item(GTK_TOOLBAR(main_toolbar), 0, "Undo last action", 0, pixmapwid, GTK_SIGNAL_FUNC(undo_last_cb), 0); 
   pixmapwid = gnome_stock_pixmap_widget_at_size(window, GNOME_STOCK_PIXMAP_REDO, 21, 21);
   main_toolbar_redo = gtk_toolbar_append_item(GTK_TOOLBAR(main_toolbar), 0, "Redo last action", 0, pixmapwid, GTK_SIGNAL_FUNC(redo_last_cb), 0); 
   gtk_toolbar_append_space(GTK_TOOLBAR(main_toolbar));
   pixmapwid = gnome_stock_pixmap_widget_at_size(window, GNOME_STOCK_PIXMAP_CUT, 21, 21);
   main_toolbar_cut = gtk_toolbar_append_item(GTK_TOOLBAR(main_toolbar), 0, "Cut selected text", 0, pixmapwid, GTK_SIGNAL_FUNC(cut_text_cb), 0); 
   pixmapwid = gnome_stock_pixmap_widget_at_size(window, GNOME_STOCK_PIXMAP_COPY, 21, 21);
   main_toolbar_copy = gtk_toolbar_append_item(GTK_TOOLBAR(main_toolbar), 0, "Copy selected text", 0, pixmapwid, GTK_SIGNAL_FUNC(copy_text_cb), 0); 
   pixmapwid = gnome_stock_pixmap_widget_at_size(window, GNOME_STOCK_PIXMAP_PASTE, 21, 21);
   main_toolbar_paste = gtk_toolbar_append_item(GTK_TOOLBAR(main_toolbar), 0, "Paste text from clipboard", 0, pixmapwid, GTK_SIGNAL_FUNC(paste_text_cb), 0); 
   gtk_toolbar_append_space(GTK_TOOLBAR(main_toolbar));
   pixmapwid = gnome_stock_pixmap_widget_at_size(window, GNOME_STOCK_PIXMAP_PROPERTIES, 21, 21);
   main_toolbar_properties = gtk_toolbar_append_item(GTK_TOOLBAR(main_toolbar), 0, "Edit the properties of a file", 0, pixmapwid, GTK_SIGNAL_FUNC(edit_properties), 0); 
   gnome_app_set_toolbar(GNOME_APP(window), GTK_TOOLBAR(main_toolbar));

#ifdef USE_PYTHON
   command_box = gtk_hbox_new(FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(command_box), 3);
   gtk_widget_show(command_box);
   label = gtk_label_new("Command:");
   gtk_box_pack_start(GTK_BOX(command_box), label, FALSE, FALSE, 5); 
   gtk_widget_show(label);

   python_combo = gtk_combo_new();
   gtk_widget_set_usize(python_combo, 400, 23);
   gtk_combo_disable_activate(GTK_COMBO(python_combo));
   gtk_box_pack_start(GTK_BOX(command_box), python_combo, TRUE, TRUE, 0);
   python_entry = GTK_COMBO(python_combo)->entry;
   gtk_signal_connect(GTK_OBJECT(python_entry), "activate", GTK_SIGNAL_FUNC(execute_macro_cb), NULL);
   command_combo_set_history(build_glist_from_file("command.hist", general_preferences.history));
   gtk_widget_show(python_combo);

   macro_button = gtk_button_new_with_label("Exec");
   gtk_signal_connect(GTK_OBJECT(macro_button), "clicked", GTK_SIGNAL_FUNC(execute_macro_cb), NULL);
   gtk_box_pack_start(GTK_BOX(command_box), macro_button, FALSE, FALSE, 0);
   gtk_widget_show(macro_button);
   gtk_widget_set_sensitive(macro_button, FALSE);
   gtk_signal_connect(GTK_OBJECT(python_entry), "changed", GTK_SIGNAL_FUNC(check_macro_button_cb), macro_button);
   gnome_app_add_docked(GNOME_APP(app), command_box, "Command Entry", GNOME_DOCK_ITEM_BEH_EXCLUSIVE, GNOME_DOCK_TOP, 3, 2, 0);
#endif

   gtk_drag_dest_set(window, (GtkDestDefaults) (GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP), drop_types, n_drop_types, GDK_ACTION_COPY);
   gtk_signal_connect(GTK_OBJECT(window), "drag_data_received", GTK_SIGNAL_FUNC(dnd_drop), 0);
}

static void dnd_drop(GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *selection_data, guint info, guint time, gpointer data)
{
   GList *names, *list;
   gchar *string;
   if(info == TARGET_URI_LIST)
   {
      list = gnome_uri_list_extract_filenames((gchar *)selection_data->data);
      for(names = list; names; names = names->next)
      {
         string = (gchar *)names->data;
         if(check_if_file_exists(string))
         {
            open_file_real((gchar *)names->data);
         }
         else
         {
            read_error(string);
         }
      }
      gnome_uri_list_free_strings(list);
   }
}