#ifndef WITH_REGEX
#define WITH_REGEX
#endif
#include "gdseditor.h"
#include "../gtkextext/gtkextext.h"

static void gds_editor_init(GdsEditor *gds_editor);
static void gds_editor_class_init(GdsEditorClass *klass);
GtkWidget *gds_editor_new();
static void gds_editor_destroy(GtkObject *object);
static gint button_press_cb(GtkWidget *widget, GdkEventButton *event);
static gint key_press_cb(GtkWidget *widget, GdkEventKey *event, GdsEditor *gds_editor);
static void property_mark(GtkExText *text, GtkExTextProperty *prop);
gint property_text_remove(GtkExText *text, GtkExTextProperty *prop, gint start, gint end);
gint property_text_insert(GtkExText *text, GtkExTextProperty *prop, gint start, gint end);
void check_pattern(GtkExText *text, gint start, gint end, GtkExTextProperty *prev);
void check_embedded(GtkExText *text, gint start, gint end, GtkExTextProperty *prev);
gint get_syntax_end(GtkExText *text, gint pos, GdsEditorSyntaxEntry *table, GtkExTextMatch *m);
void check_syntax(GtkExText *text, gint i, gint end);

static GtkWidgetClass *parent_class = NULL;

GtkType gds_editor_get_type(void)
{
   static GtkType gds_editor_type=0;
   if(!gds_editor_type)
   {
      static const GtkTypeInfo gds_editor_info = 
      {	
         "GdsEditor",
         sizeof(GdsEditor),
         sizeof(GdsEditorClass),
         (GtkClassInitFunc) gds_editor_class_init,
         (GtkObjectInitFunc) gds_editor_init,
         NULL,
         NULL,
         (GtkClassInitFunc)NULL,
      };
      gds_editor_type = gtk_type_unique(GTK_TYPE_EXTEXT, &gds_editor_info);
   }
   return(gds_editor_type);
}

static void gds_editor_class_init(GdsEditorClass *klass)
{
   GtkObjectClass *object_class;
   object_class = (GtkObjectClass*)klass;
   parent_class = gtk_type_class(GTK_TYPE_EXTEXT);
   object_class->destroy = gds_editor_destroy;
}

static void gds_editor_init(GdsEditor *gds_editor)
{
   gds_editor->changed = FALSE;
   gds_editor->highlight = FALSE;
}

GtkWidget* gds_editor_new()
{
   GdsEditor *gds_editor;
   GtkObject *hadj;
   hadj = gtk_adjustment_new(0, 0, 256, 20, 1, 80);
   gds_editor = (GdsEditor *) gtk_widget_new(GDS_TYPE_EDITOR, "hadjustment", hadj, "vadjustment", NULL, NULL);
   gtk_signal_connect(GTK_OBJECT(gds_editor), "property_text_remove", GTK_SIGNAL_FUNC(property_text_remove), NULL);
   gtk_signal_connect(GTK_OBJECT(gds_editor), "property_text_insert", GTK_SIGNAL_FUNC(property_text_insert), NULL);
   gtk_signal_connect(GTK_OBJECT(gds_editor), "property_mark", GTK_SIGNAL_FUNC(property_mark), NULL);
   gtk_signal_connect(GTK_OBJECT(gds_editor), "button_press_event", GTK_SIGNAL_FUNC(button_press_cb), 0);
   gtk_signal_connect(GTK_OBJECT(gds_editor), "key_press_event", GTK_SIGNAL_FUNC(key_press_cb), gds_editor);
   gtk_signal_connect(GTK_OBJECT(gds_editor), "changed", GTK_SIGNAL_FUNC(gds_editor_set_changed), 0);
   return GTK_WIDGET(gds_editor);
}

static void gds_editor_destroy(GtkObject *object)
{
   GdsEditor *gds_editor;
   g_return_if_fail(object != NULL);
   g_return_if_fail(GDS_IS_EDITOR(object));
   gds_editor = GDS_EDITOR(object);
	GTK_OBJECT_CLASS(parent_class)->destroy(object);
}

void gds_editor_set_changed(GdsEditor *gds_editor, gpointer data)
{
   g_return_if_fail(gds_editor != NULL);
   g_return_if_fail(GDS_IS_EDITOR(gds_editor));
   gds_editor->changed++;
   gtk_signal_emit_by_name(GTK_OBJECT(gds_editor), "move_to_column", 0, NULL);
}

gboolean gds_editor_changed(GdsEditor *gds_editor)
{
   g_return_val_if_fail(gds_editor != NULL, FALSE);
   g_return_val_if_fail(GDS_IS_EDITOR(gds_editor), FALSE);
   if(gds_editor->changed) return(TRUE);
   return(FALSE);
}

void gds_editor_set_colors(GdsEditor *gds_editor, GdkColor *fg, GdkColor *bg)
{
   GdkColormap *colormap;
   g_return_if_fail(gds_editor != NULL);
   g_return_if_fail(GDS_IS_EDITOR(gds_editor));
   if(GTK_WIDGET_REALIZED(gds_editor))
   {
      colormap = gdk_window_get_colormap(GTK_EXTEXT(gds_editor)->text_area);
      gdk_color_alloc(colormap, fg);
      colormap = gdk_window_get_colormap(GTK_WIDGET(gds_editor)->window);
      gdk_color_alloc(colormap, fg);
      colormap = gdk_window_get_colormap(GTK_EXTEXT(gds_editor)->text_area);
      gdk_color_alloc(colormap, bg);
      gdk_window_set_background(GTK_EXTEXT(gds_editor)->text_area, bg);
      colormap = gdk_window_get_colormap(GTK_WIDGET(gds_editor)->window);
      gdk_color_alloc(colormap, bg);
      gdk_window_set_background(GTK_WIDGET(gds_editor)->window, bg);
   }
   if(GTK_EXTEXT(gds_editor)->gc)
   {
      gdk_gc_destroy(GTK_EXTEXT(gds_editor)->gc);
      GTK_EXTEXT(gds_editor)->gc = gdk_gc_new (GTK_EXTEXT(gds_editor)->text_area);
      gdk_gc_set_exposures(GTK_EXTEXT(gds_editor)->gc, TRUE);
      gdk_gc_set_foreground(GTK_EXTEXT(gds_editor)->gc, fg);
      gdk_gc_set_background(GTK_EXTEXT(gds_editor)->gc, bg);
   }
   if(GTK_EXTEXT(gds_editor)->bg_gc)
   {
      gdk_gc_destroy(GTK_EXTEXT(gds_editor)->bg_gc);
      if(GTK_WIDGET(gds_editor)->style->bg_pixmap[GTK_STATE_NORMAL])
      {
         GdkGCValues values;    
         values.tile = GTK_WIDGET(gds_editor)->style->bg_pixmap[GTK_STATE_NORMAL];
         values.fill = GDK_TILED;
         GTK_EXTEXT(gds_editor)->bg_gc = gdk_gc_new_with_values(GTK_EXTEXT(gds_editor)->text_area, &values, GDK_GC_FILL | GDK_GC_TILE);
      }
      else
      {
         GTK_EXTEXT(gds_editor)->bg_gc = gdk_gc_new(GTK_EXTEXT(gds_editor)->text_area);
      }
      gdk_gc_set_foreground(GTK_EXTEXT(gds_editor)->bg_gc, bg);
      gdk_gc_set_background(GTK_EXTEXT(gds_editor)->bg_gc, bg);
   }
   gtk_extext_style_insert(GTK_EXTEXT(gds_editor), "Default", GTK_EXTEXT(gds_editor)->font, fg, bg);
   gtk_widget_queue_draw(GTK_WIDGET(gds_editor));
}

void gds_editor_set_selection_colors(GdsEditor *gds_editor, GdkColor *fg, GdkColor *bg)
{
   g_return_if_fail(gds_editor != NULL);
   g_return_if_fail(GDS_IS_EDITOR(gds_editor));
   gtk_extext_style_insert(GTK_EXTEXT(gds_editor), "Selected", GTK_EXTEXT(gds_editor)->font, fg, bg);
}

void gds_editor_set_font(GdsEditor *gds_editor, GdkFont *font)
{
   g_return_if_fail(gds_editor != NULL);
   g_return_if_fail(GDS_IS_EDITOR(gds_editor));
   if(font == NULL) return;
   if(GTK_EXTEXT(gds_editor)->font)
      gdk_font_unref(GTK_EXTEXT(gds_editor)->font);
   GTK_EXTEXT(gds_editor)->font = font;
   gdk_font_ref(font);
   gtk_extext_style_insert(GTK_EXTEXT(gds_editor), "Default", font, &GTK_WIDGET(gds_editor)->style->text[GTK_STATE_NORMAL], &GTK_WIDGET(gds_editor)->style->bg[GTK_STATE_NORMAL]);
   gtk_extext_style_insert(GTK_EXTEXT(gds_editor), "Selected", font, &GTK_WIDGET(gds_editor)->style->text[GTK_STATE_SELECTED], &GTK_WIDGET(gds_editor)->style->bg[GTK_STATE_SELECTED]);
}

void gds_editor_set_highlight(GdsEditor *text, gboolean value)
{
   g_return_if_fail(text != NULL);
   g_return_if_fail(GDS_IS_EDITOR(text));
  
   text->highlight = value;
   gtk_extext_property_remove_all(GTK_EXTEXT(text),0,GTK_EXTEXT(text)->length,NULL);
   if(text->highlight) check_syntax(GTK_EXTEXT(text), 0, GTK_EXTEXT(text)->length);
}

void gds_editor_highlight(GdsEditor *text)
{
   if(text->highlight)
   {
      gtk_extext_freeze(GTK_EXTEXT(text));
      gtk_extext_property_remove_all(GTK_EXTEXT(text), 0, GTK_EXTEXT(text)->length, NULL);
      check_syntax(GTK_EXTEXT(text), 0, GTK_EXTEXT(text)->length);
      gtk_extext_thaw(GTK_EXTEXT(text));
   }
}

static gint button_press_cb(GtkWidget *widget, GdkEventButton *event)
{
   GtkExText *text;
   gboolean handled = FALSE;
   gboolean shift_state;
   gboolean control_state;
   gint skip_size = 10;

   shift_state = event->state & GDK_SHIFT_MASK;
   control_state = event->state & GDK_CONTROL_MASK;  

   if(shift_state) skip_size *= 4;
   if(control_state) skip_size *= 8;  

   text = (GtkExText *)widget;

   if(!GTK_WIDGET_HAS_FOCUS(widget))
      gtk_widget_grab_focus(widget);

   else if(event->type == GDK_BUTTON_PRESS  && event->button == 4)
   {
      if((text->line_pos_index - skip_size) >= 0)
         gtk_extext_set_line(text, text->line_pos_index - skip_size);
      else
         gtk_extext_set_line(text, 0);
      handled = TRUE;
   }
   else if(event->type == GDK_BUTTON_PRESS && event->button == 5)
   {
      if((text->line_pos_index + skip_size) <= text->line_count)
         gtk_extext_set_line(text, text->line_pos_index + skip_size);
      else
         gtk_extext_set_line(text, text->line_count);
      handled = TRUE;
   }

   if(handled) gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "button_press_event");
   return(handled);
}

static gint key_press_cb(GtkWidget *widget, GdkEventKey *event, GdsEditor *gds_editor)
{
   GtkExText *text;
   GtkEditable *editable;
   gint curpos;
   gboolean handled = FALSE;
   gboolean shift_state;
   gboolean control_state;
   gint key;
   
   shift_state = event->state & GDK_SHIFT_MASK;
   control_state = event->state & GDK_CONTROL_MASK;  
   key = event->keyval;
   
   text = (GtkExText *)widget;
   editable = (GtkEditable *)widget;
   curpos = gtk_extext_get_position(editable);
     
   if(event->keyval == 'x' && control_state)
   {
      gtk_editable_cut_clipboard(editable);
      editable->has_selection = FALSE;
      editable->selection_start_pos = -1;
      editable->selection_end_pos = -1;
      handled = TRUE;
   }
   else if(event->keyval == 'e' && control_state)
   {
      gchar search_char = 0, base_char = 0, cur_char = 0;
      gint position = 0;
      gint addition = -1;
      gint counter = 0;
      gboolean found = FALSE;
      position = curpos;
      base_char = gtk_extext_get_char_at_pos(text, position);
      switch((int)base_char)
      {
         case '{': addition = 1;
                   search_char = '}';
                   break;
         case '(': addition = 1;
                   search_char = ')';
                   break;
         case '[': addition = 1;
                   search_char = ']';
                   break;
         case '<': addition = 1;
                   search_char = '>';
                   break;
         case '}': addition = -1;
                   search_char = '{';
                   break;
         case ')': addition = -1;
                   search_char = '(';
                   break;
         case ']': addition = -1;
                   search_char = '[';
                   break;
         case '>': addition = -1;
                   search_char = '<';
                   break;
         default : addition = 1;
                   search_char = base_char;
                   break;
      }
      while(position >= 0 && position <= text->length)
      {
         position += addition;
         cur_char = gtk_extext_get_char_at_pos(text, position);
         if(cur_char == search_char && !counter)
         {
            found = TRUE;
            break;
         }
         if(cur_char == base_char)
            counter++;
         else if(cur_char == search_char)
            counter--;
      }
      if(found)
      {
         gtk_extext_set_position(editable, position);
         handled = TRUE;
      }
   }
   else if(event->keyval == 'z' && control_state)
   {
      editable->has_selection = FALSE;
      editable->selection_start_pos = -1;
      editable->selection_end_pos = -1;
      if(!gtk_extext_undo_is_empty(text))
      {
         gtk_extext_undo(text);
         gds_editor->changed -= 2;
         gtk_signal_emit_by_name(GTK_OBJECT(gds_editor), "move_to_column", 0, NULL);
      }
      handled = TRUE;
   }
   else if(event->keyval == 'r' && control_state)
   {
      editable->has_selection = FALSE;
      editable->selection_start_pos = -1;
      editable->selection_end_pos = -1;
      if(!gtk_extext_redo_is_empty(text))
      {
         gtk_extext_redo(text);
         gtk_signal_emit_by_name(GTK_OBJECT(gds_editor), "move_to_column", 0, NULL);
      }
      handled = TRUE;
   }
   else if(event->keyval == GDK_Delete)
   {
      if(shift_state && editable->has_selection)
         gtk_editable_cut_clipboard(editable);
      else if(editable->has_selection)
         gtk_editable_delete_text(GTK_EDITABLE(text), editable->selection_start_pos, editable->selection_end_pos);
      else
         gtk_editable_delete_text(GTK_EDITABLE(text), GTK_EDITABLE(text)->current_pos, GTK_EDITABLE(text)->current_pos+1);
      editable->has_selection = FALSE;
      editable->selection_start_pos = -1;
      editable->selection_end_pos = -1;
      handled = TRUE;
   }
   if(handled) gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
   return(handled);
}

gboolean gds_editor_compile_regex(const gchar *pattern, Regex *regex)
{
   memset (&regex->buf, 0, sizeof(regex->buf));
   regex->buf.translate = NULL;
   regex->buf.fastmap = g_malloc (256);
   regex->buf.allocated = 0;
   regex->buf.buffer = NULL;
   regex->buf.can_be_null = 0;	/* so we wont allow that in patterns! */
   regex->buf.no_sub = 0;
   if(re_compile_pattern(pattern, strlen(pattern), &regex->buf) == 0)
   {
      if(re_compile_fastmap(&regex->buf) != 0)
      {
         g_warning("IMPORTANT REGEX FAILED TO CREATE FASTMAP\n");
         g_free (regex->buf.fastmap);
         regex->buf.fastmap = NULL;
      }
      return TRUE;
   }
   else
   {
      g_warning("IMPORTANT REGEX FAILED TO COMPILE\n");
      return FALSE;
   }
}

void property_mark(GtkExText *text,GtkExTextProperty *prop)
{
   if(!prop) return;
}

gint property_text_remove(GtkExText *text, GtkExTextProperty *prop, gint start, gint end)
{
   GtkExTextLineData *slinedataptr, *elinedataptr;
   gint diff, e;
   g_return_val_if_fail(text != NULL, FALSE);
   g_return_val_if_fail(GTK_IS_EXTEXT(text), FALSE);
   if(!GDS_EDITOR(text)->highlight) return FALSE;
   if(end > text->length)
   {
      gtk_extext_property_remove_all(text, text->length, end, NULL);
      end=text->length;
   }  
   diff = end - start;
   e = end;
   if((prop = gtk_extext_property_get_at_pos(text, start, NULL)) && ((gint)prop->startpos <= start) && ((gint)(prop->endpos-diff) >= end))
   {
      start = prop->startpos;
      end = prop->endpos-diff;
   }
   else
   {
      slinedataptr = gtk_extext_get_line_by_char_pos(text, start);
      start = slinedataptr && slinedataptr->startpos < start ? slinedataptr->startpos : start;
      g_free(slinedataptr);
      elinedataptr = gtk_extext_get_line_by_char_pos(text, end);
      end = elinedataptr && elinedataptr->endpos > end ? elinedataptr->endpos : end;
      g_free(elinedataptr);
   }
   g_print("Remove Highlight - (%d, %d)\n", start, end);
   gtk_extext_property_move_all(text, e, -diff, NULL);
   gtk_extext_property_remove_all(text, start, end, NULL);
   check_syntax(text, start, end);
   gtk_signal_emit_stop_by_name(GTK_OBJECT(text), "property_text_remove");
   return(TRUE);
}

gint property_text_insert(GtkExText *text, GtkExTextProperty *prop, gint start, gint end)
{
   gint diff, s;
   GtkExTextLineData *slinedataptr, *elinedataptr;
   g_return_val_if_fail(text != NULL, FALSE);
   g_return_val_if_fail(GDS_IS_EDITOR(text), FALSE);
 
   if(!GDS_EDITOR(text)->highlight) return(FALSE);
   s = start;
   diff = end - start;
   if((prop = gtk_extext_property_get_at_pos(text, start, NULL)) && ((gint)prop->startpos < start) && ((gint)(prop->endpos+diff) >= end))
   {
      start = prop->startpos;
      end = prop->endpos+diff < text->length ? prop->endpos+diff : text->length;
   }
   else
   {
      slinedataptr = gtk_extext_get_line_by_char_pos(text, start);
      start = slinedataptr && slinedataptr->startpos < start ? slinedataptr->startpos : start;
      g_free(slinedataptr);
      elinedataptr = gtk_extext_get_line_by_char_pos(text, end);
      end = elinedataptr && elinedataptr->endpos > end ? elinedataptr->endpos : end;
      g_free(elinedataptr);
   }
   g_print("Insert Highlight - (%d, %d)\n", start, end);
   gtk_extext_property_move_all(text, s, diff, NULL);
   gtk_extext_property_remove_all(text, start, end, NULL);
   check_syntax(text, start, end);
   gtk_signal_emit_stop_by_name(GTK_OBJECT(text), "property_text_insert");
   return(TRUE);
}

void check_syntax(GtkExText *text, gint start, gint end)
{
   gint s,i;
   gboolean found = FALSE;
   GList *list;
   GtkExTextMatch m;
   GtkExTextProperty *prev;
   GdsEditorHighlightTable *table;
   GdsEditorSyntaxEntry *sentry;
   table = GDS_EDITOR(text)->syntax_table;
   if(!table)
   {
      check_embedded(text, start, end, NULL);
      return;
   }
   list = g_list_first(table->entries);
   i = start;
   prev = text->scroll_line_start->property_first;
   if(end > text->length)
   {
      g_warning("THIS IS BUGGY check_syntax end [%d] is more than text->length [%d]\n", end, text->length);
      end=text->length;
   }
   while(i < end)
   {
      s = gtk_extext_regex_search(text, i, &table->regex_all, TRUE, &m);
      if(s < 0 ||  s > end) break;
      if(i < s) check_embedded(text, i, s, prev);
      i = m.endpos;
      list = g_list_first(table->entries);
      while(list)
      {    
         sentry = (GdsEditorSyntaxEntry *)list->data;
         if(gtk_extext_regex_match(text, s, &sentry->regex_start) > 0 && GTK_EXTEXT_INDEX(text, s-1) != '\\')
         {
            if(get_syntax_end(text, i, sentry, &m) >= 0)
               i = m.endpos;
            else
               i = text->length;
            prev = gtk_extext_property_insert(text, sentry->key, s, i, GINT_TO_POINTER(SYNTAX_TABLE), PROPERTY_REMOVEALL, prev);
            found = TRUE;
            break;
         }
         list = g_list_next(list);
      }
      if(!found) i++;
   }
   if(i < end) check_embedded(text, i, end, prev);
}

void check_embedded(GtkExText *text, gint start, gint end, GtkExTextProperty *prev)
{
   gint i, j, pos;
   gint startpos = 0;
   gint endpos = 0;
   GList *list;
   GdsEditorHighlightTable *table;
   GdsEditorEmbeddedEntry *embedded;
   table = GDS_EDITOR(text)->embedded_table;
   if(!table)
   {
   	check_pattern(text, start, end, prev);
   	return;
   }
   list = g_list_first(table->entries);
   if(end > text->length)
   {
      g_warning("THIS IS BUGGY! end [%d] is more than text->length [%d]\n", end, text->length);
      end = text->length;
   }
   prev = NULL;
   i = start;
   while(i < end)
   {
      list = g_list_first(table->entries);
      while(list)
      {
         embedded = (GdsEditorEmbeddedEntry *)list->data;
         if((pos = gtk_extext_regex_match(text, i, &embedded->start) > 0))
         {
            startpos = endpos = i + pos;
            while(endpos < text->length)
            {
               if((pos = gtk_extext_regex_match(text, endpos, &embedded->end)) > 0)
               {
                  j = endpos + pos;
                  while(startpos < endpos)
                  {
                     //i = j-1;
                     if((pos = gtk_extext_regex_match(text, startpos, &embedded->regex)) > 0)
                     {
                        prev = gtk_extext_property_insert(text, embedded->key, startpos, startpos + pos, GINT_TO_POINTER(EMBEDDED_TABLE), PROPERTY_INSERT, prev);
                        startpos += pos;
                     }
                     else startpos++;
                  }
                  break;
               }
               endpos++;
            }
         }
         list = g_list_next(list);
      }
      i++;
   }
   check_pattern(text, start, end, prev);
}

void check_pattern(GtkExText *text, gint start, gint end, GtkExTextProperty *prev)
{
   gint i;
   gint pos = 0;
   gboolean found;
   GList *list;
   GdsEditorHighlightTable *table;
   GdsEditorPatternEntry *pentry;
 
   table = GDS_EDITOR(text)->pattern_table;
   if(!table) return;
   list = g_list_first(table->entries);
   if(end > text->length)
   {
      g_warning("THIS IS BUGGY! end [%d] is more than text->length [%d]\n", end, text->length);
      end = text->length;
   }
   prev = NULL;
   i = start;
   while(i < end)
   {
      list = g_list_first(table->entries);
      found = FALSE;
      while(list)
      {
         pentry = (GdsEditorPatternEntry *)list->data;      
         if((pos = gtk_extext_regex_match(text, i, &pentry->regex) ) > 0 && end >= i + pos)
         {
            prev = gtk_extext_property_insert(text, pentry->key, i, i + pos, GINT_TO_POINTER(PATTERN_TABLE), PROPERTY_INSERT,prev);
       	i += pos;
            found = TRUE;
            break;
         }
         list = g_list_next(list);
      }
      if(!found) i++;
   }
   gtk_widget_queue_draw(GTK_WIDGET(text));
}

gint get_syntax_end(GtkExText *text, gint pos, GdsEditorSyntaxEntry *table, GtkExTextMatch *m)
{
   int ret;
   do {
      ret = gtk_extext_regex_search(text, m->endpos, &table->regex_end, TRUE, m);
      if(ret < 0) return -1;
   } while(m->endpos && GTK_EXTEXT_INDEX(text, m->endpos-2) == '\\' );
   return(ret);
}

void gds_editor_install_table(GdsEditor *text, GdsEditorHighlightTable *table)
{
   GList *cur;
   GdsEditorSyntaxEntry *sentry;
   GdsEditorPatternEntry *pentry;
   GdsEditorEmbeddedEntry *eentry;
 
   g_return_if_fail(text != NULL);
   g_return_if_fail(GDS_IS_EDITOR(text));
   g_return_if_fail(table != NULL);
   
   if(table->type == SYNTAX_TABLE)
   {
      text->syntax_table = table;
      cur = g_list_first(table->entries);
      while(cur)
      {
         sentry = (GdsEditorSyntaxEntry *)cur->data;
         gtk_extext_style_insert(GTK_EXTEXT(text), sentry->key, sentry->font, sentry->fg_color, sentry->bg_color);
         cur = g_list_next(cur);
      }
   }
   else if(table->type == PATTERN_TABLE)
   {
      text->pattern_table = table;
      cur=g_list_first(table->entries);
      while(cur)
      {
         pentry = (GdsEditorPatternEntry*)cur->data;
         gtk_extext_style_insert(GTK_EXTEXT(text),pentry->key, pentry->font,pentry->fg_color, pentry->bg_color);
         cur = g_list_next(cur);
      }
   }
   else if(table->type == EMBEDDED_TABLE)
   {
      text->embedded_table = table;
      cur=g_list_first(table->entries);
      while(cur)
      {
         eentry = (GdsEditorEmbeddedEntry*)cur->data;
         gtk_extext_style_insert(GTK_EXTEXT(text), eentry->key, eentry->font, eentry->fg_color, eentry->bg_color);
         cur = g_list_next(cur);
      }
   }
   else
   {
      g_warning("Table could not be inserted.\n");
   }
}

GdsEditorHighlightTable *gds_editor_syntax_table_new(GList *entries)
{
   gchar *regs;
   GList *cur;
   GdsEditorHighlightTable *table;
   GdsEditorSyntaxEntry *syntax;
   g_return_val_if_fail(entries!=NULL,NULL);
   g_return_val_if_fail(g_list_length(entries),NULL);
 
   table = g_malloc0(sizeof(GdsEditorHighlightTable));
   table->type = SYNTAX_TABLE;    
   table->entries = entries;
   regs = g_malloc0(1024);
   cur = g_list_first(entries);   
   while(cur)
   {
      syntax = (GdsEditorSyntaxEntry *)cur->data;
      strcat(regs,syntax->start);
      cur = g_list_next(cur);
      if(cur) strcat(regs,"\\|");    
   }
   gds_editor_compile_regex(regs,&table->regex_all);
   g_free(regs);
   return(table);
}

GdsEditorHighlightTable *gds_editor_pattern_table_new(GList *entries)
{
   GdsEditorHighlightTable *table;
   g_return_val_if_fail(entries!=NULL,NULL);
   g_return_val_if_fail(g_list_length(entries),NULL);

   table = g_malloc0(sizeof(GdsEditorHighlightTable));  
   table->type = PATTERN_TABLE;    
   table->entries = entries;
   return(table);
}

GdsEditorHighlightTable *gds_editor_embedded_table_new(GList *entries)
{
   GdsEditorHighlightTable *table;
   g_return_val_if_fail(entries != NULL, NULL);
   g_return_val_if_fail(g_list_length(entries), NULL);

   table = g_malloc0(sizeof(GdsEditorHighlightTable));  
   table->type = EMBEDDED_TABLE;    
   table->entries = entries;
   return(table);
}

GList *gds_editor_syntax_entry_new(GList *entries, gchar *name, gchar *start, gchar *end, GdkFont *font,GdkColor *fg, GdkColor*bg)
{
   GdsEditorSyntaxEntry *syntax;
   syntax = g_malloc0(sizeof(GdsEditorSyntaxEntry));
   syntax->start = g_strdup(start);
   strncpy(syntax->key,name,32);  
   syntax->font = font;
   syntax->bg_color = bg;
   syntax->fg_color = fg;
   gds_editor_compile_regex(start,&syntax->regex_start);
   gds_editor_compile_regex(end,&syntax->regex_end);
   return(g_list_append(entries, syntax));
}

GList *gds_editor_pattern_entry_new(GList *entries, gchar *name, gchar *start, GdkFont *font, GdkColor *fg, GdkColor*bg)
{
   GdsEditorPatternEntry *pattern;
   pattern = g_malloc0(sizeof(GdsEditorPatternEntry));
   strncpy(pattern->key, name, 32);
   pattern->font = font;
   pattern->bg_color = bg;
   pattern->fg_color = fg;
   gds_editor_compile_regex(start, &pattern->regex);
   return(g_list_append(entries, pattern));
}

GList *gds_editor_embedded_entry_new(GList *entries, gchar *name, gchar *start, gchar *end, gchar *words, GdkFont *font, GdkColor *fg, GdkColor*bg)
{
   GdsEditorEmbeddedEntry *embedded;
   embedded = g_malloc0(sizeof(GdsEditorEmbeddedEntry));
   strncpy(embedded->key, name, 32);
   embedded->font = font;
   embedded->bg_color = bg;
   embedded->fg_color = fg;
   gds_editor_compile_regex(start, &embedded->start);
   gds_editor_compile_regex(end, &embedded->end);
   gds_editor_compile_regex(words, &embedded->regex);
   return(g_list_append(entries, embedded));
}

void gds_editor_table_free(GdsEditorHighlightTable *table)
{
   GdsEditorSyntaxEntry *syntax;
   GdsEditorPatternEntry *pattern;
   GdsEditorEmbeddedEntry *embedded;
   
   GList *cur;
   cur = g_list_first(table->entries);
   while(cur)
   {
      if(table->type == SYNTAX_TABLE)
      {
         syntax = (GdsEditorSyntaxEntry *)cur->data;
         g_free(syntax->regex_start.buf.fastmap);
         syntax->regex_start.buf.fastmap = NULL;
         regfree(&syntax->regex_start.buf);
         g_free(syntax->regex_end.buf.fastmap);
         syntax->regex_end.buf.fastmap = NULL;
         regfree(&syntax->regex_end.buf);
      }
      else if(table->type == PATTERN_TABLE)
      {
         pattern = (GdsEditorPatternEntry *)cur->data;
         g_free(pattern->regex.buf.fastmap);
         pattern->regex.buf.fastmap = NULL;
         regfree(&pattern->regex.buf);
      }
      if(table->type == EMBEDDED_TABLE)
      {
         embedded = (GdsEditorEmbeddedEntry *)cur->data;
         g_free(embedded->start.buf.fastmap);
         embedded->start.buf.fastmap = NULL;
         regfree(&embedded->start.buf);
         g_free(embedded->end.buf.fastmap);
         embedded->end.buf.fastmap = NULL;
         regfree(&embedded->end.buf);
         g_free(embedded->regex.buf.fastmap);
         embedded->regex.buf.fastmap = NULL;
         regfree(&embedded->regex.buf);
      }
      g_free(cur->data);
      cur=g_list_next(cur);
   }
   g_list_free(table->entries);
   g_free(table);
}
